/***************************************************************************
Demo fr das Punktmatrix Display DLG7137

Hardware: ATtiny2313A @1MHz, default Fusebits: Low= 0x64, alle anderen= 0xFF
PB 0-6: Datenbus Ausgang zu D0-D6 (Pins 8..14) des Displays
PB 7	/LT Lamptest: Alle Punkte aller Displays leuchten gleichzeitig
PD 0-2	binre Adresse 0-7 zur Digit Auswahl /CE via 74LS138
PD 3	/WR Impuls parallel an alle 8 Digits

(c) TN, 12/03/2025
****************************************************************************/

#include <avr/io.h>
#include <util/delay.h>

uint8_t Buchstabe;

void PutChar (uint8_t pos, uint8_t ch)
{
//	Datenbyte anlegen
	ch = (ch | 0b10000000);			// Bit 7 setzen (LED-Test inaktiv)
	PORTB = ch;

//	Bits 0-2 = Adresse zum 74LS138, Bit 3 setzen (/WR inaktiv)
	pos = (pos | 0b00001000);
	PORTD = pos;					// Adresse anlegen => Display 0-7 jeweiliges /CE auswhlen

//	-_- Schreibimpuls erzeugen
	PORTD &= ~(1 << PD3);			// WR = 0 (aktiv) => Daten bernehmen
	PORTD |=  (1 << PD3);			// WR = 1 (passiv)
}

////////
//Main//
////////
int main(void)
{
// 	Setzen der Port-Richtungsregister
	DDRD = 0x0F;					// PortD unteres Nibble Steuersignale (Ausgnge)
	DDRB = 0xFF;					// PortB 7 Bit Datenbus Ausgang zum Display, Bit 8 = /LT

//	Initialisieren der Ports
	PORTD= 0b00001000;				// Bits 0-2:Digitauswahl, 0 = ganz rechts, PD3 /WR = 1 = inaktiv
	PORTB= 0b00000000;				// 7 Bit Daten = 00h, LED-Test = 0 = aktiv
	_delay_ms (500); 				// LED-Test kurz anzeigen

	PutChar (7, 'D');				// Testausgabe
	PutChar (6, 'L');
	PutChar (5, 'G');
	PutChar (4, '7');
	PutChar (3, '1');
	PutChar (2, '3');
	PutChar (1, '7');
	PutChar (0, '!');
	_delay_ms (500);

	Buchstabe = 0x20;				// 1. Zeichen = Blank
	while(1)						// Zeichensatz auf rechtem Digit endlos ausgeben
	{
		PutChar (0, Buchstabe);
		_delay_ms (500);
		Buchstabe++;
		if (Buchstabe == 0x80) Buchstabe = 0x20;
	}
}
